]> sigrok.org Git - libsigrok.git/blobdiff - filter.c
Code drop from DreamSourceLabs first source release.
[libsigrok.git] / filter.c
index fc6de634f1b276e4a1cd3a2c3d656e589f7e86fb..11cad5c38096771ad442568773b4c24c90d30830 100644 (file)
--- a/filter.c
+++ b/filter.c
@@ -1,5 +1,5 @@
 /*
- * This file is part of the sigrok project.
+ * This file is part of the libsigrok project.
  *
  * Copyright (C) 2010-2012 Bert Vermeulen <bert@biot.com>
  *
 #include <stdlib.h>
 #include <stdint.h>
 #include <string.h>
+#include <glib.h>
 #include "libsigrok.h"
 #include "libsigrok-internal.h"
 
+/* Message logging helpers with subsystem-specific prefix string. */
+#define LOG_PREFIX "filter: "
+#define sr_log(l, s, args...) sr_log(l, LOG_PREFIX s, ## args)
+#define sr_spew(s, args...) sr_spew(LOG_PREFIX s, ## args)
+#define sr_dbg(s, args...) sr_dbg(LOG_PREFIX s, ## args)
+#define sr_info(s, args...) sr_info(LOG_PREFIX s, ## args)
+#define sr_warn(s, args...) sr_warn(LOG_PREFIX s, ## args)
+#define sr_err(s, args...) sr_err(LOG_PREFIX s, ## args)
+
+/**
+ * @file
+ *
+ * Helper functions to filter out unused probes from samples.
+ */
+
+/**
+ * @defgroup grp_filter Probe filter
+ *
+ * Helper functions to filter out unused probes from samples.
+ *
+ * @{
+ */
+
 /**
  * Remove unused probes from samples.
  *
@@ -56,8 +80,8 @@
  *                     The requested unit size must be big enough to hold as
  *                     much data as is specified by the number of enabled
  *                     probes in 'probelist'.
* @param probelist Pointer to a list of probe numbers, numbered starting
*                  from 0. The list is terminated with -1.
  * @param data_in Pointer to the input data buffer. Must not be NULL.
  * @param length_in The input data length (>= 1), in number of bytes.
  * @param data_out Variable which will point to the newly allocated buffer
  *         or SR_ERR_ARG upon invalid arguments.
  *         If something other than SR_OK is returned, the values of
  *         out_unitsize, data_out, and length_out are undefined.
+ *
+ * @since 0.1.0 (but the API changed in 0.2.0)
  */
-SR_API int sr_filter_probes(int in_unitsize, int out_unitsize,
-                           const int *probelist, const uint8_t *data_in,
+SR_API int sr_filter_probes(unsigned int in_unitsize, unsigned int out_unitsize,
+                           const GArray *probe_array, const uint8_t *data_in,
                            uint64_t length_in, uint8_t **data_out,
                            uint64_t *length_out)
 {
        unsigned int in_offset, out_offset;
-       int num_enabled_probes, out_bit, i;
+       int *probelist, out_bit;
+       unsigned int i;
        uint64_t sample_in, sample_out;
 
-       if (!probelist) {
-               sr_err("filter: %s: probelist was NULL", __func__);
+       if (!probe_array) {
+               sr_err("%s: probe_array was NULL", __func__);
                return SR_ERR_ARG;
        }
+       probelist = (int *)probe_array->data;
 
        if (!data_in) {
-               sr_err("filter: %s: data_in was NULL", __func__);
+               sr_err("%s: data_in was NULL", __func__);
                return SR_ERR_ARG;
        }
 
        if (!data_out) {
-               sr_err("filter: %s: data_out was NULL", __func__);
+               sr_err("%s: data_out was NULL", __func__);
                return SR_ERR_ARG;
        }
 
        if (!length_out) {
-               sr_err("filter: %s: length_out was NULL", __func__);
+               sr_err("%s: length_out was NULL", __func__);
                return SR_ERR_ARG;
        }
 
-       num_enabled_probes = 0;
-       for (i = 0; probelist[i] != -1; i++)
-               num_enabled_probes++;
-
        /* Are there more probes than the target unit size supports? */
-       if (num_enabled_probes > out_unitsize * 8) {
-               sr_err("filter: %s: too many probes (%d) for the target unit "
-                      "size (%d)", __func__, num_enabled_probes, out_unitsize);
+       if (probe_array->len > out_unitsize * 8) {
+               sr_err("%s: too many probes (%d) for the target unit "
+                      "size (%d)", __func__, probe_array->len, out_unitsize);
                return SR_ERR_ARG;
        }
 
        if (!(*data_out = g_try_malloc(length_in))) {
-               sr_err("filter: %s: data_out malloc failed", __func__);
+               sr_err("%s: data_out malloc failed", __func__);
                return SR_ERR_MALLOC;
        }
 
-       if (num_enabled_probes == in_unitsize * 8) {
+       if (probe_array->len == in_unitsize * 8) {
                /* All probes are used -- no need to compress anything. */
                memcpy(*data_out, data_in, length_in);
                *length_out = length_in;
@@ -129,7 +153,7 @@ SR_API int sr_filter_probes(int in_unitsize, int out_unitsize,
        while (in_offset <= length_in - in_unitsize) {
                memcpy(&sample_in, data_in + in_offset, in_unitsize);
                sample_out = out_bit = 0;
-               for (i = 0; probelist[i] != -1; i++) {
+               for (i = 0; i < probe_array->len; i++) {
                        if (sample_in & (1 << (probelist[i])))
                                sample_out |= (1 << out_bit);
                        out_bit++;
@@ -142,3 +166,5 @@ SR_API int sr_filter_probes(int in_unitsize, int out_unitsize,
 
        return SR_OK;
 }
+
+/** @} */
@@ -56,8 +80,8 @@
  *                     much data as is specified by the number of enabled
  *                     probes in 'probelist'.
- * @param probelist Pointer to a list of probe numbers, numbered starting
- *                  from 0. The list is terminated with -1.
+ * @param probe_array Pointer to a list of probe numbers, numbered starting
+ *                    from 0. The list is terminated with -1.
  * @param data_in Pointer to the input data buffer. Must not be NULL.
  * @param length_in The input data length (>= 1), in number of bytes.